import os
import sys
import subprocess as sp

import pytest


def pytest_configure(config):
    config.addinivalue_line(
        "markers", "embedded: the test create an embedded executable"
    )


skip_if_win32 = pytest.mark.skipif(
    "sys.platform == 'win32'", reason="skipping Posix tests on Windows"
)

skip_if_macos = pytest.mark.skipif(
    "sys.platform == 'darwin'", reason="skipping test on macOS"
)

skip_if_pypy = pytest.mark.skipif(
    "'__pypy__' in sys.builtin_module_names", reason="skipping test on pypy"
)

skip_if_no_proc_env = pytest.mark.skipif(
    "not os.path.exists('/proc/self/environ')",
    reason="'/proc/self/environ' not available",
)

skip_if_no_proc_cmdline = pytest.mark.skipif(
    "not os.path.exists('/proc/%s/cmdline' % os.getpid())",
    reason="'/proc/PID/cmdline' not available",
)

skip_if_no_proc_tasks = pytest.mark.skipif(
    "not os.path.exists('/proc/self/task')",
    reason="'/proc/self/task' not available",
)


@pytest.fixture(scope="session")
def pyrun(pyconfig):
    """
    Build the pyrun executable and return its path
    """
    # poor man's make
    here = os.path.abspath(os.path.dirname(__file__))
    ver2 = "%s.%s" % sys.version_info[:2]
    source = os.path.join(here, "pyrun.c")
    target = os.path.join(here, f"pyrun{ver2}")
    if (
        os.path.exists(target)
        and os.stat(target).st_mtime > os.stat(source).st_mtime
    ):
        return target

    cmdline = ["cc"]  # big punt
    cmdline.extend(pyconfig("includes"))
    cmdline.extend(["-o", target, source])
    cmdline.extend(pyconfig("ldflags"))
    cmdline.append(f"-L{pyconfig('prefix')[0]}/lib")
    sp.check_call(cmdline)
    return target


@pytest.fixture(scope="session")
def pyconfig():
    """Return the result of 'python-config --opt' as a list of strings"""
    pyexe = os.path.realpath(sys.executable)
    ver2 = "%s.%s" % sys.version_info[:2]
    for name in (f"python{ver2}-config", "python3-config", "python-config"):
        pyconfexe = os.path.join(os.path.dirname(pyexe), name)
        if os.path.exists(pyconfexe):
            break
    else:
        pytest.fail(
            f"can't find python-config from executable {sys.executable}"
        )

    # Travis' Python 3.8 is not built with --embed
    help = sp.check_output([pyconfexe, "--help"])
    has_embed = b"--embed" in help

    def pyconfig_func(opt):
        cmdline = [pyconfexe, f"--{opt}"]
        if has_embed:
            cmdline.append("--embed")
        bout = sp.check_output(cmdline)
        out = bout.decode(
            sys.getfilesystemencoding()  # sounds like a good bet
        )
        return out.split()

    return pyconfig_func


@pytest.fixture(scope="session")
def spt_directory():
    """
    Where is the setproctitle module installed?
    """
    rv = run_script(
        """
import os
import setproctitle
print(os.path.dirname(os.path.dirname(setproctitle.__file__)))
"""
    )
    return rv.rstrip()


@pytest.fixture(scope="function")
def tmp_pypath(monkeypatch, tmp_path):
    """
    return a tmp directory which has been added to the python path
    """
    monkeypatch.setenv(
        "PYTHONPATH",
        str(tmp_path) + os.pathsep + os.environ.get("PYTHONPATH", ""),
    )
    return tmp_path


def run_script(script=None, args=None, executable=None, env=None):
    """run a script in a separate process.

    if the script completes successfully, return its ``stdout``,
    else fail the test.
    """
    if executable is None:
        executable = sys.executable

    cmdline = str(executable)
    if args:
        cmdline = cmdline + " " + args

    proc = sp.Popen(
        cmdline,
        stdin=sp.PIPE,
        stdout=sp.PIPE,
        stderr=sp.PIPE,
        env=env,
        shell=True,
        close_fds=True,
    )

    out, err = proc.communicate(script and script.encode())
    if 0 != proc.returncode:
        if out:
            print(out.decode("utf8", "replace"), file=sys.stdout)
        if err:
            print(err.decode("utf8", "replace"), file=sys.stderr)
        pytest.fail("test script failed")

    # Py3 subprocess generates bytes strings.
    out = out.decode()

    return out



"""setproctitle module unit test.

Use pytest to run this test suite.

The tests are executed in external processes: setproctitle should
never be imported directly from here.

Copyright (c) 2009-2021 Daniele Varrazzo <daniele.varrazzo@gmail.com>
"""

import os
import re
import sys
import string
import subprocess as sp

import pytest

from .conftest import run_script, skip_if_no_proc_cmdline, skip_if_no_proc_env
from .conftest import skip_if_macos, skip_if_pypy, skip_if_win32

pytestmark = [skip_if_win32]


def test_runner():
    """Test the script execution method."""
    rv = run_script(
        """
print(10 + 20)
"""
    )
    assert rv == "30\n"


@pytest.mark.skipif(
    'sys.platform == "darwin" and (os.environ.get("CIBW_TEST_COMMAND") or sys.version_info >= (3, 11))',
    reason="f*cked up binary name",
)
def test_init_getproctitle():
    """getproctitle() returns a sensible value at initial call."""
    rv = run_script(
        """
import setproctitle
print(setproctitle.getproctitle())
""",
        args="-u",
    )
    assert rv == sys.executable + " -u\n"


def test_setproctitle():
    """setproctitle() can set the process title, duh."""
    rv = run_script(
        r"""
import setproctitle
setproctitle.setproctitle('Hello, world!')

import os
print(os.getpid())
# ps can fail on kfreebsd arch
# (http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=460331)
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
"""
    )
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])
    title = _clean_up_title(pids[pid])
    assert title == "Hello, world!"


@pytest.mark.skipif('sys.platform != "darwin"', reason="Mac only test")
def test_setproctitle_darwin():
    """Mac Activity monitor shows correct info"""
    rv = run_script(
        r"""
import setproctitle
setproctitle.setproctitle('QwErTyZxCvB')

import os
print(os.popen("lsappinfo find name=QwErTyZxCvB 2> /dev/null").read())
"""
    )
    m = re.search(r'''ASN:.*"QwErTyZxCvB"''', rv)
    assert m


def test_prctl():
    """Check that prctl is called on supported platforms."""
    linux_version = []
    if sys.platform in ("linux", "linux2"):
        try:
            f = os.popen("uname -r")
            name = f.read()
            f.close()
        except Exception:
            pass
        else:
            linux_version = list(
                map(int, re.search("[.0-9]+", name).group().split(".")[:3])
            )

    if linux_version < [2, 6, 9]:
        pytest.skip("syscall not supported")

    rv = run_script(
        r"""
import setproctitle
setproctitle.setproctitle('Hello, prctl!')
print(open('/proc/self/status').read())
"""
    )
    status = dict([r.split(":", 1) for r in rv.splitlines() if ":" in r])
    assert status["Name"].strip() == "Hello, prctl!"


def test_getproctitle():
    """getproctitle() can read the process title back."""
    rv = run_script(
        r"""
import setproctitle
setproctitle.setproctitle('Hello, world!')
print(setproctitle.getproctitle())
"""
    )
    assert rv == "Hello, world!\n"


def test_kwarg():
    """setproctitle() supports keyword args."""
    rv = run_script(
        r"""
import setproctitle
setproctitle.setproctitle(title='Hello, world!')
print(setproctitle.getproctitle())
"""
    )
    assert rv == "Hello, world!\n"


def test_environ():
    """Check that clobbering environ didn't break env."""
    rv = run_script(
        r"""
import setproctitle
setproctitle.setproctitle('Hello, world! ' + 'X' * 1024)

# set a new env variable, update another one
import os
os.environ['TEST_SETENV'] = "setenv-value"
os.environ['PATH'] = os.environ.get('PATH', '') \
        + os.pathsep + "fakepath"

# read the environment from a spawned process inheriting the
# updated env
newenv = dict([r.split("=",1)
        for r in os.popen("env").read().splitlines()
        if '=' in r])

print(setproctitle.getproctitle())
print(newenv['TEST_SETENV'])
print(newenv['PATH'])
"""
    )

    title, test, path = rv.splitlines()
    assert title.startswith("Hello, world! XXXXX"), title
    assert test == "setenv-value"
    assert path.endswith("fakepath"), path


def test_issue_8(tmp_pypath):
    """Test that the module works with 'python -m'."""
    module = "spt_issue_8"
    with open(tmp_pypath / f"{module}.py", "w") as f:
        f.write(
            r"""
import setproctitle
setproctitle.setproctitle("Hello, module!")

import os
print(os.getpid())
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
            """
        )

    rv = run_script(args="-m " + module)
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    title = _clean_up_title(pids[pid])
    assert title == "Hello, module!"


def test_large_cmdline(tmp_pypath):
    """Test with a 64KB command line."""
    module = "longargs"
    with open(tmp_pypath / f"{module}.py", "w") as f:
        f.write(
            r"""
import setproctitle
setproctitle.setproctitle("Hello, long!")

import os
print(os.getpid())
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
            """
        )

    rv = run_script(args=f"-m {module} {' '.join(['X' * 1024] * 64)}")
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    title = _clean_up_title(pids[pid])
    assert title == "Hello, long!"


def test_unicode():
    """Title can contain unicode characters."""
    snowman = "\u2603"
    try:
        snowman.encode(sys.getdefaultencoding())
    except UnicodeEncodeError:
        pytest.skip(
            "default encoding '%s' can't deal with snowmen"
            % sys.getdefaultencoding()
        )

    try:
        snowman.encode(sys.getfilesystemencoding())
    except UnicodeEncodeError:
        pytest.skip(
            "file system encoding '%s' can't deal with snowmen"
            % sys.getfilesystemencoding()
        )

    rv = run_script(
        r"""
snowman = u'\u2603'

import setproctitle
setproctitle.setproctitle("Hello, " + snowman + "!")

import os
import locale
from subprocess import Popen, PIPE
print(os.getpid())
proc = Popen("ps -x -o pid,command 2> /dev/null", shell=True,
    close_fds=True, stdout=PIPE, stderr=PIPE)
buf = proc.stdout.read()
print(buf.decode(locale.getpreferredencoding(), 'replace'))
"""
    )
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    snowmen = [
        "\u2603",  # ps supports unicode
        r"\M-b\M^X\M^C",  # ps output on BSD
        r"M-bM^XM^C",  # ps output on some Darwin < 11.2
        "\ufffdM^XM^C",  # ps output on Darwin 11.2
    ]
    title = _clean_up_title(pids[pid])
    for snowman in snowmen:
        if title == "Hello, " + snowman + "!":
            break
    else:
        pytest.fail("unexpected ps output: %r" % title)


def test_weird_args():
    """No problem with encoded arguments."""
    euro = "\u20ac"
    snowman = "\u2603"
    try:
        rv = run_script(
            r"""
import setproctitle
setproctitle.setproctitle("Hello, weird args!")

import os
print(os.getpid())
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
""",
            args=" ".join(["-", "hello", euro, snowman]),
        )
    except TypeError:
        pytest.skip("apparently we can't pass unicode args to a program")

    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    title = _clean_up_title(pids[pid])
    assert title == "Hello, weird args!"


def test_weird_path(tmp_path, spt_directory):
    """No problem with encoded argv[0] path."""
    _check_4388()
    euro = "\u20ac"
    snowman = "\u2603"
    dir = tmp_path / euro / snowman
    try:
        os.makedirs(dir)
    except UnicodeEncodeError:
        pytest.skip("file system doesn't support unicode")

    exc = dir / os.path.basename(sys.executable)
    os.symlink(sys.executable, exc)

    rv = run_script(
        f"""
import sys
sys.path.insert(0, {repr(spt_directory)})

import setproctitle
setproctitle.setproctitle("Hello, weird path!")

import os
print(os.getpid())
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
""",
        args=" ".join(["-", "foo", "bar", "baz"]),
        executable=exc,
    )
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    title = _clean_up_title(pids[pid])
    assert title == "Hello, weird path!"


@pytest.mark.embedded
@skip_if_pypy
@skip_if_macos
@skip_if_no_proc_cmdline
def test_embedded(pyrun, spt_directory):
    """Check the module works with embedded Python."""
    rv = run_script(
        f"""
import sys
sys.path.insert(0, {spt_directory!r})
import setproctitle
setproctitle.setproctitle("Hello, embedded!")

import os
print(os.getpid())
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
""",
        executable=pyrun,
    )
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    title = _clean_up_title(pids[pid])
    assert title == "Hello, embedded!"


@pytest.mark.embedded
@skip_if_pypy
@skip_if_macos
@skip_if_no_proc_cmdline
def test_embedded_many_args(pyrun, spt_directory):
    """Check more complex cmdlines are handled in embedded env too."""
    rv = run_script(
        f"""
import sys
sys.path.insert(0, {spt_directory!r})
import setproctitle
setproctitle.setproctitle("Hello, embedded!")

import os
print(os.getpid())
print(os.popen("ps -x -o pid,command 2> /dev/null").read())
""",
        executable=pyrun,
        args=" ".join(["foo", "bar", "baz"]),
    )
    lines = [line for line in rv.splitlines() if line]
    pid = lines.pop(0)
    pids = dict([r.strip().split(None, 1) for r in lines])

    title = _clean_up_title(pids[pid])
    assert title == "Hello, embedded!"


@skip_if_no_proc_env
def test_noenv():
    """Check that SPT_NOENV avoids clobbering environ."""
    env = os.environ.copy()
    env["SPT_TESTENV"] = "testenv"
    rv = run_script(
        """
import os
os.environ['SPT_NOENV'] = "1"

cmdline_len = len(open('/proc/self/cmdline').read())
print(cmdline_len)
print('SPT_TESTENV=testenv' in open('/proc/self/environ').read())

import setproctitle
setproctitle.setproctitle('X' * cmdline_len * 10)

title = open('/proc/self/cmdline').read().rstrip()
print(title)
print(len(title))

print('SPT_TESTENV=testenv' in open('/proc/self/environ').read())
    """,
        env=env,
    )
    lines = rv.splitlines()
    cmdline_len = int(lines[0])
    assert lines[1] == "True", "can't verify testenv"
    title = lines[2]
    assert "XXX" in _clean_up_title(title), "title not set as expected"
    title_len = int(lines[3])
    assert lines[4] == "True", "env has been clobbered"
    assert (
        title_len <= cmdline_len
    ), "title (len {title_len}) not limited to argv (len {cmdline_len})"


@skip_if_no_proc_env
def test_large_env(monkeypatch):
    """Check that large environment doesn't get clobbered."""
    monkeypatch.setenv("SPT_NOENV", "1")
    for c in string.ascii_uppercase:
        monkeypatch.setenv(
            f"{c}_TEST_ENV", "X" * (ord(c) - ord("A") + 1) * 1024
        )

    rv = run_script(
        r"""\
import sys
with open("/proc/self/environ", "rb") as f:
    env1 = f.read()
    sys.stdout.buffer.write(env1)

sys.stdout.buffer.write(b"\n-----8<-----\n")

import setproctitle
setproctitle.setproctitle("hello")

with open("/proc/self/environ", "rb") as f:
    env2 = f.read()
    sys.stdout.buffer.write(env2)
"""
    )
    parts = rv.split("\n-----8<-----\n")
    for i, part in enumerate(parts):
        parts[i] = dict(
            var.split("=", 1) for var in part.split("\0") if "=" in var
        )

    assert parts[0] == parts[1]


def test_clear_segfault():
    run_script(
        r"""\
import os
from setproctitle import setproctitle
os.environ.clear()
setproctitle("Test")
"""
    )


def test_fork_segfault():
    run_script(
        """\
import multiprocessing as mp
from setproctitle import setproctitle

def foo():
    setproctitle('title in child')

setproctitle('title in parent')
mp.set_start_method("fork")
p = mp.Process(target=foo)
p.start()
p.join()
assert p.exitcode == 0, f"p.exitcode is {p.exitcode}"
"""
    )


def test_thread_fork_segfault():
    run_script(
        """\
import multiprocessing as mp
from threading import Thread
from setproctitle import setproctitle

def foo():
    setproctitle("title in child")

def thread():
    global p
    p = mp.Process(target=foo)
    p.start()
    p.join()

p = None
mp.set_start_method("fork")
t = Thread(target=thread)
t.start()
t.join()
assert p.exitcode == 0, f"p.exitcode is {p.exitcode}"
"""
    )


# Support functions


def _clean_up_title(title):
    """Clean up a string from the prefix added by the platform."""
    # BSD's setproctitle decorates the title with the process name.
    if "bsd" in sys.platform:
        procname = os.path.basename(sys.executable)
        title = " ".join([t for t in title.split(" ") if procname not in t])

    return title


def _check_4388():
    """Check if the system is affected by bug #4388.

    If positive, unicode chars in the cmdline are not reliable,
    so bail out.

    see: http://bugs.python.org/issue4388
    """
    if sys.getfilesystemencoding() == "ascii":
        # in this case the char below would get translated in some
        # inconsistent way.
        # I'm not getting why the FS encoding is involved in process
        # spawning, the whole story just seems a gigantic can of worms.
        return

    p = sp.Popen([sys.executable, "-c", "ord('\xe9')"], stderr=sp.PIPE)
    p.communicate()
    if p.returncode:
        pytest.skip("bug #4388 detected")



import os  # noqa
import sys  # noqa

from .conftest import run_script, skip_if_win32, skip_if_no_proc_tasks

pytestmark = [skip_if_win32, skip_if_no_proc_tasks]


def test_thread_title_unchanged():
    rv = run_script(
        """
from glob import glob

def print_stuff():
    for fn in sorted(glob("/proc/self/task/*/comm")):
        with open(fn) as f:
            print(f.readline().rstrip())

print_stuff()
print("---")
import setproctitle
print_stuff()
print("---")
print(setproctitle.getthreadtitle())
"""
    )
    before, after, gtt = rv.split("---\n")
    assert before == after
    assert before == gtt


def test_set_thread_title():
    run_script(
        """
from glob import glob
import setproctitle
setproctitle.setthreadtitle("hello" * 10)

(fn,) = glob("/proc/self/task/*/comm")
with open(fn) as f:
    assert f.read().rstrip() == "hello" * 3
"""
    )


def test_set_threads_title():
    run_script(
        """
import time
import threading
from glob import glob

(fn,) = glob("/proc/self/task/*/comm")
with open(fn) as f:
    orig = f.read().rstrip()

import setproctitle

def worker(title):
    setproctitle.setthreadtitle(title)
    while 1:
        time.sleep(1)

t1 = threading.Thread(target=worker, args=('reader',), daemon=True)
t2 = threading.Thread(target=worker, args=('writer',), daemon=True)
t1.start()
t2.start()

comms = []
for fn in glob("/proc/self/task/*/comm"):
    with open(fn) as f:
        comms.append(f.read().rstrip())

comms.sort()
assert comms == sorted([orig, "reader", "writer"])
"""
    )



#!/usr/bin/env python
"""
setproctitle setup script.

Copyright (c) 2009-2021 Daniele Varrazzo <daniele.varrazzo@gmail.com>
"""

import re
import sys

from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext

with open("pkg/setproctitle/__init__.py") as f:
    data = f.read()
    m = re.search(r"""(?m)^__version__\s*=\s*['"]([^'"]+)['"]""", data)
    if not m:
        raise Exception(f"cannot find version in {f.name}")
    VERSION = m.group(1)

define_macros = {}
define_macros["SPT_VERSION"] = VERSION

platform_sources = []

if sys.platform.startswith("linux"):
    define_macros["HAVE_SYS_PRCTL_H"] = 1

elif sys.platform == "darwin":
    # __darwin__ symbol is not defined; __APPLE__ is instead.
    define_macros["__darwin__"] = 1
    platform_sources.append("src/darwin_set_process_name.c")

elif "bsd" in sys.platform:  # OMG, how many of them are?
    define_macros["BSD"] = 1
    define_macros["HAVE_SETPROCTITLE"] = 1
    define_macros["HAVE_PS_STRING"] = 1

# NOTE: the module may work on HP-UX using pstat
# thus setting define_macros['HAVE_SYS_PSTAT_H']
# see http://www.noc.utoronto.ca/~mikep/unix/HPTRICKS
# But I have none handy to test with.

mod_spt = Extension(
    "setproctitle._setproctitle",
    define_macros=list(define_macros.items()),
    sources=[
        "src/setproctitle.c",
        "src/spt_debug.c",
        "src/spt_setup.c",
        "src/spt_status.c",
        "src/spt_strlcpy.c",
    ]
    + platform_sources,
)

# Try to include the long description in the setup
kwargs = {}
try:
    kwargs["long_description"] = (
        open("README.rst").read() + "\n" + open("HISTORY.rst").read()
    )
    kwargs["long_description_content_type"] = "text/x-rst"
except Exception:
    pass

classifiers = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Programming Language :: C
Programming Language :: Python :: 3
Programming Language :: Python :: 3.7
Programming Language :: Python :: 3.8
Programming Language :: Python :: 3.9
Programming Language :: Python :: 3.10
Programming Language :: Python :: 3.11
Programming Language :: Python :: 3.12
Operating System :: POSIX :: Linux
Operating System :: POSIX :: BSD
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Topic :: Software Development
""".splitlines()


class BuildError(Exception):
    pass


class setproctitle_build_ext(build_ext):
    def run(self) -> None:
        try:
            super().run()
        except Exception as e:
            print(f"Failed to build C module: {e}", file=sys.stderr)
            raise BuildError(str(e))

    def build_extension(self, ext):
        try:
            super().build_extension(ext)
        except Exception as e:
            print(
                f"Failed to build extension {ext.name}: {e}", file=sys.stderr
            )
            raise BuildError(str(e))


def do_build(with_extension):
    ext_modules = [mod_spt] if with_extension else []
    setup(
        name="setproctitle",
        description="A Python module to customize the process title",
        version=VERSION,
        author="Daniele Varrazzo",
        author_email="daniele.varrazzo@gmail.com",
        url="https://github.com/dvarrazzo/py-setproctitle",
        download_url="http://pypi.python.org/pypi/setproctitle/",
        license="BSD-3-Clause",
        platforms=["GNU/Linux", "BSD", "MacOS X", "Windows"],
        python_requires=">=3.7",
        classifiers=classifiers,
        packages=["setproctitle"],
        package_dir={"setproctitle": "pkg/setproctitle"},
        ext_modules=ext_modules,
        package_data={"setproctitle": ["py.typed"]},
        extras_require={"test": ["pytest"]},
        cmdclass={"build_ext": setproctitle_build_ext},
        zip_safe=False,
        **kwargs,
    )


try:
    do_build(with_extension=True)
except BuildError:
    do_build(with_extension=False)



import pytest
import setproctitle
import sys  # noqa

skip_if_not_win32 = pytest.mark.skipif(
    "sys.platform != 'win32'", reason="Windows only test"
)

pytestmark = [skip_if_not_win32]


def test_setproctitle():
    title = "setproctitle_test"
    setproctitle.setproctitle(title)
    assert title == setproctitle.getproctitle()


def test_setthreadtitle():
    title = "setproctitle_test"
    # This is currently a no-op on Windows. Let's make sure
    # that at least it doesn't error out.
    setproctitle.setthreadtitle(title)



from .conftest import run_script, skip_if_win32


@skip_if_win32
def test_no_import_side_effect():
    """
    Check that importing the module doesn't cause side effects.
    """
    rv = run_script(
        """
import os

def print_stuff():
    for fn in "cmdline status comm".split():
        if os.path.exists(f"/proc/self/{fn}"):
            with open(f"/proc/self/{fn}") as f:
                print(f.readline().rstrip())

print_stuff()
print("---")
import setproctitle
print_stuff()
"""
    )
    before, after = rv.split("---\n")
    assert before == after


def test_version():
    """Test that the module has a version"""
    rv = run_script(
        """
import setproctitle
print(setproctitle.__version__)
"""
    )
    assert rv


def test_c_extension_built():
    """Test that the C extension was built"""
    rv = run_script(
        """
from setproctitle import _setproctitle
"""
    )
    assert rv == ""
